Package: ggfortify
Type: Package
Title: Data Visualization Tools for Statistical Analysis Results
Version: 0.4.3
Date: 2018-03-02
Authors@R: c(
  person("Masaaki", "Horikoshi", role = c("aut"),
         email = "sinhrks@gmail.com"),
  person("Yuan", "Tang", role = c("aut", "cre"),
         email = "terrytangyuan@gmail.com"),
  person("Austin", "Dickey", role = c("ctb")),
  person("Matthias", "Grenié", role = c("ctb")),
  person("Ryan", "Thompson", role = c("ctb")),
  person("Luciano", "Selzer", role = c("ctb")),
  person("Dario", "Strbenac", role = c("ctb")),
  person("Kirill", "Voronin", role = c("ctb"))
  )
Maintainer: Yuan Tang <terrytangyuan@gmail.com>
URL: https://github.com/sinhrks/ggfortify
BugReports: https://github.com/sinhrks/ggfortify/issues
Encoding: UTF-8
Description: Unified plotting tools for statistics commonly used, such as GLM,
    time series, PCA families, clustering and survival analysis. The package offers
    a single plotting interface for these analysis results and plots in a unified
    style using 'ggplot2'.
License: GPL-2
VignetteBuilder: knitr
Depends: methods, ggplot2 (>= 2.0.0)
Imports: dplyr (>= 0.3), tidyr, gridExtra, grid, scales, stringr,
        tibble
Suggests: testthat, cluster, changepoint, dlm, fGarch, forecast,
        ggrepel, glmnet, grDevices, KFAS, knitr, lintr, mapdata, MASS,
        MSwM, nlme, raster, ROCR, sp, stats, strucchange, survival,
        timeSeries, tseries, utils, vars, xts, zoo, lfda
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2018-03-02 14:11:51 UTC; terrytangyuan
Author: Masaaki Horikoshi [aut],
  Yuan Tang [aut, cre],
  Austin Dickey [ctb],
  Matthias Grenié [ctb],
  Ryan Thompson [ctb],
  Luciano Selzer [ctb],
  Dario Strbenac [ctb],
  Kirill Voronin [ctb]
Repository: CRAN
Date/Publication: 2018-03-02 15:52:34 UTC
Built: R 3.3.2; ; 2018-03-23 11:50:53 UTC; windows
