# Generated by roxygen2: do not edit by hand

S3method(autoplot,Arima)
S3method(autoplot,HoltWinters)
S3method(autoplot,KFS)
S3method(autoplot,Line)
S3method(autoplot,Lines)
S3method(autoplot,MSM.lm)
S3method(autoplot,Polygon)
S3method(autoplot,Polygons)
S3method(autoplot,RasterBrick)
S3method(autoplot,RasterCommon)
S3method(autoplot,RasterLayer)
S3method(autoplot,RasterStack)
S3method(autoplot,SpatialCommon)
S3method(autoplot,SpatialLines)
S3method(autoplot,SpatialLinesDataFrame)
S3method(autoplot,SpatialPoints)
S3method(autoplot,SpatialPointsDataFrame)
S3method(autoplot,SpatialPolygons)
S3method(autoplot,SpatialPolygonsDataFrame)
S3method(autoplot,aareg)
S3method(autoplot,acf)
S3method(autoplot,ar)
S3method(autoplot,basis)
S3method(autoplot,bats)
S3method(autoplot,breakpoints)
S3method(autoplot,clara)
S3method(autoplot,cpt)
S3method(autoplot,cv.glmnet)
S3method(autoplot,decomposed.ts)
S3method(autoplot,density)
S3method(autoplot,dist)
S3method(autoplot,dlmFiltered)
S3method(autoplot,ets)
S3method(autoplot,fGARCH)
S3method(autoplot,factanal)
S3method(autoplot,fanny)
S3method(autoplot,forecast)
S3method(autoplot,fracdiff)
S3method(autoplot,ggmultiplot)
S3method(autoplot,ggplot)
S3method(autoplot,glm)
S3method(autoplot,glmnet)
S3method(autoplot,irts)
S3method(autoplot,kmeans)
S3method(autoplot,lfda)
S3method(autoplot,list)
S3method(autoplot,lm)
S3method(autoplot,map)
S3method(autoplot,matrix)
S3method(autoplot,nnetar)
S3method(autoplot,partition)
S3method(autoplot,pca_common)
S3method(autoplot,performance)
S3method(autoplot,prcomp)
S3method(autoplot,princomp)
S3method(autoplot,silhouette)
S3method(autoplot,spec)
S3method(autoplot,stepfun)
S3method(autoplot,stl)
S3method(autoplot,survfit)
S3method(autoplot,timeSeries)
S3method(autoplot,ts)
S3method(autoplot,tsmodel)
S3method(autoplot,varprd)
S3method(autoplot,xts)
S3method(autoplot,zooreg)
S3method(fitted,ar)
S3method(fortify,Arima)
S3method(fortify,HoltWinters)
S3method(fortify,KFS)
S3method(fortify,MSM.lm)
S3method(fortify,RasterBrick)
S3method(fortify,RasterCommon)
S3method(fortify,RasterLayer)
S3method(fortify,RasterStack)
S3method(fortify,SpatialCommon)
S3method(fortify,SpatialLines)
S3method(fortify,SpatialPoints)
S3method(fortify,SpatialPointsDataFrame)
S3method(fortify,aareg)
S3method(fortify,acf)
S3method(fortify,ar)
S3method(fortify,basis)
S3method(fortify,bats)
S3method(fortify,breakpoints)
S3method(fortify,breakpointsfull)
S3method(fortify,clara)
S3method(fortify,cpt)
S3method(fortify,cv.glmnet)
S3method(fortify,decomposed.ts)
S3method(fortify,density)
S3method(fortify,dist)
S3method(fortify,dlmFiltered)
S3method(fortify,ets)
S3method(fortify,fGARCH)
S3method(fortify,factanal)
S3method(fortify,fanny)
S3method(fortify,forecast)
S3method(fortify,fracdiff)
S3method(fortify,glmnet)
S3method(fortify,irts)
S3method(fortify,kmeans)
S3method(fortify,lfda)
S3method(fortify,list)
S3method(fortify,matrix)
S3method(fortify,nnetar)
S3method(fortify,pam)
S3method(fortify,partition)
S3method(fortify,performance)
S3method(fortify,prcomp)
S3method(fortify,princomp)
S3method(fortify,silhouette)
S3method(fortify,spec)
S3method(fortify,stepfun)
S3method(fortify,stl)
S3method(fortify,survfit)
S3method(fortify,table)
S3method(fortify,timeSeries)
S3method(fortify,ts)
S3method(fortify,tsmodel)
S3method(fortify,varprd)
S3method(grid.draw,ggmultiplot)
S3method(residuals,ar)
export(fortify_map)
export(ggbiplot)
export(ggcpgram)
export(ggdistribution)
export(ggfreqplot)
export(ggtsdiag)
export(rbind_ts)
export(unscale)
exportClasses(ggmultiplot)
import(dplyr)
import(ggplot2)
import(methods)
importFrom(grid,grid.draw)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(scales,percent)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,predict)
importFrom(stats,reshape)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,weights)
importFrom(stringr,str_wrap)
importFrom(tibble,as_tibble)
importFrom(tidyr,gather)
importFrom(utils,installed.packages)
