<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{\u4e00\u884cR\u4ee3\u7801\u6765\u5b9e\u73b0\u7e41\u7410\u7684\u53ef\u89c6\u5316 (Introduction in Chinese)}
-->

author: "唐源"

[ggfortify](https://github.com/sinhrks/ggfortify) 有着简单易用的统一的界面来用一行代码来对许多受欢迎的R软件包结果进行二维可视化的一个R工具包。这让许多的统计学家以及数据科学家省去了许多繁琐和重复的过程，不用对结果进行任何处理就能以 `{ggplot}` 的风格画出好看的图，大大地提高了工作的效率。

虽然ggfortify已经在[CRAN](https://cran.r-project.org/package=ggfortify)上，但是由于最近很多的功能都还在快速增加，还是推荐大家从[Github](https://github.com/sinhrks/ggfortify)上下载和安装
```
library(devtools)
install_github('sinhrks/ggfortify')
library(ggfortify)
```

```{r global_options, include=FALSE}
library(knitr)
opts_chunk$set(fig.width=6, fig.height=3, fig.path='figures/pca-', warning=FALSE)
```
接下来我将简单介绍一下怎么用 `{ggplot2}` 和 `{ggfortify}` 来很快的对PCA, clustering, 以及LFDA的结果进行可视化。然后将简单介绍用 `{ggfortify}` 来对时间序列进行迅速的可视化。

# PCA (主成分分析)

`{ggfortify}` 使 `{ggplot2}` 知道怎么诠释PCA对象. 加载好 `{ggfortify}`包之后, 你可以对`stats::prcomp` 和 `stats::princomp`对象使用 `ggplot2::autoplot`。

```{r, message = FALSE}
library(ggfortify)
df <- iris[c(1, 2, 3, 4)]
autoplot(prcomp(df))
```

你还可以选择数据中的一列来给画出的点按类别自动分颜色。输入`help(autoplot.prcomp)` 可以了解到更多的其他选择。

```{r, message = FALSE}
autoplot(prcomp(df), data = iris, colour = 'Species')
```

比如说给定`label = TRUE` 可以给每个点加上标识(以`rownames`为标准)，也可以调整标识的大小。

```{r, message = FALSE}
autoplot(prcomp(df), data = iris, colour = 'Species', label = TRUE, label.size = 3)
```

给定 `shape = FALSE` 可以让所有的点消失，只留下标识，这样可以让图更清晰，辨识度更大。

```{r, message = FALSE}
autoplot(prcomp(df), data = iris, colour = 'Species', shape = FALSE, label.size = 3)
```

给定 `loadings = TRUE` 可以很快的画出特征向量。

```{r, message = FALSE}
autoplot(prcomp(df), data = iris, colour = 'Species', loadings = TRUE)
```

同样的，你也可以显示特征向量的标识以及调整他们的大小，更多选择请参考帮助文件。

```{r, message = FALSE}
autoplot(prcomp(df), data = iris, colour = 'Species',
         loadings = TRUE, loadings.colour = 'blue',
         loadings.label = TRUE, loadings.label.size = 3)
```

# 因素分析

和PCA类似，`ggfortify` 也支持 `stats::factanal` 对象。可调的选择也很广泛。以下给出了简单的例子：

**注意** 当你使用 `factanal` 来计算分数的话，你必须给定 `scores`的值。

```{r, message = FALSE}
d.factanal <- factanal(state.x77, factors = 3, scores = 'regression')
autoplot(d.factanal, data = state.x77, colour = 'Income')
autoplot(d.factanal, label = TRUE, label.size = 3,
         loadings = TRUE, loadings.label = TRUE, loadings.label.size  = 3)
```

# K-均值

```{r, message = FALSE}

autoplot(kmeans(USArrests, 3), data = USArrests)

autoplot(kmeans(USArrests, 3), data = USArrests, label = TRUE, label.size = 3)
```

# cluster(集群)

`{ggfortify}` 也支持 `cluster::clara`, `cluster::fanny`, `cluster::pam`。

```{r, message = FALSE}
library(cluster)
autoplot(clara(iris[-5], 3))
```

给定 `frame = TRUE`，可以把`stats::kmeans` 和 `cluster::*` 的每个集群圈出来。

```{r}
autoplot(fanny(iris[-5], 3), frame = TRUE)
```

你也可以通过 `frame.type` 来选择圈的类型。更多选择请参照[`ggplot2::stat_ellipse`](http://docs.ggplot2.org/dev/stat_ellipse.html)里面的`frame.type`的`type`关键词。

```{r}
autoplot(pam(iris[-5], 3), frame = TRUE, frame.type = 'norm')
```

# lfda(Fisher局部判别分析)

[``{lfda}``](https://cran.r-project.org/package=lfda)包支持一系列的Fisher局部判别分析方法，包括半监督lfda，非线性lfda。你也可以使用`{ggfortify}`来对他们的结果进行可视化。

```{r, message = FALSE, eval = FALSE}
library(lfda)

# Fisher局部判别分析 (LFDA)
model <- lfda(iris[-5], iris[, 5], 4, metric="plain")
autoplot(model, data = iris, frame = TRUE, frame.colour = 'Species')
```

```{r, message = FALSE, eval = FALSE}
# 半监督Fisher局部判别分析 (SELF)
model <- self(iris[-5], iris[, 5], beta = 0.1, r = 3, metric="plain")
autoplot(model, data = iris, frame = TRUE, frame.colour = 'Species')
```

更多关于集群方面的可视化请参考Github上的[Vignette](https://github.com/sinhrks/ggfortify/tree/master/vignettes)或者[Rpubs](http://rpubs.com/sinhrks/plot_pca)上的例子。


# 时间序列的可视化
用 `{ggfortify}` 使时间序列的可视化变得及其简单。接下来我将给出一些简单的例子。

# ts对象

```{r, message = FALSE}
library(ggfortify)
autoplot(AirPassengers)
```

可以使用 `ts.colour` 和 `ts.linetype`来改变线的颜色和形状。更多的选择请参考 `help(autoplot.ts)`。

```{r}
autoplot(AirPassengers, ts.colour = 'red', ts.linetype = 'dashed')
```

# 多变量时间序列

```{r message = FALSE}
library(vars)
data(Canada)
autoplot(Canada)
```

使用 `facets = FALSE` 可以把所有变量画在一条轴上。

```{r}
autoplot(Canada, facets = FALSE)
```

`autoplot` 也可以理解其他的时间序列类别。可支持的R包有：

* `zoo::zooreg`
* `xts::xts`
* `timeSeries::timSeries`
* `tseries::irts`

一些例子：

```{r, message = FALSE}
library(xts)
autoplot(as.xts(AirPassengers), ts.colour = 'green')

library(timeSeries)
autoplot(as.timeSeries(AirPassengers), ts.colour = ('dodgerblue3'))
```

你也可以通过 `ts.geom` 来改变几何形状，目前支持的有 `line`, `bar` and `point`.

```{r, message = FALSE}
autoplot(AirPassengers, ts.geom = 'bar', fill = 'blue')
```

```{r, message = FALSE}
autoplot(AirPassengers, ts.geom = 'point', shape = 3)
```

# forecast包

```{r, message = FALSE}
library(forecast)
d.arima <- auto.arima(AirPassengers)
d.forecast <- forecast(d.arima, level = c(95), h = 50)
autoplot(d.forecast)
```

有很多设置可供调整：

```{r}
autoplot(d.forecast, ts.colour = 'firebrick1', predict.colour = 'red',
         predict.linetype = 'dashed', conf.int = FALSE)
```

# vars包

```{r}
library(vars)
d.vselect <- VARselect(Canada, lag.max = 5, type = 'const')$selection[1]
d.var <- VAR(Canada, p = d.vselect, type = 'const')
```

更多可设置可供调整：

```{r}
autoplot(predict(d.var, n.ahead = 50), ts.colour = 'dodgerblue4',
         predict.colour = 'blue', predict.linetype = 'dashed')
```

# changepoint包

```{r message = FALSE}
library(changepoint)
autoplot(cpt.meanvar(AirPassengers))
```


```{r}
autoplot(cpt.meanvar(AirPassengers), cpt.colour = 'blue', cpt.linetype = 'solid')
```

# strucchange包

```{r message = FALSE}
library(strucchange)
autoplot(breakpoints(Nile ~ 1), ts.colour = 'blue', ts.linetype = 'dashed',
         cpt.colour = 'dodgerblue3', cpt.linetype = 'solid')
```

# dlm包

```{r, message=FALSE, warning=FALSE}
library(dlm)
form <- function(theta){
  dlmModPoly(order = 1, dV = exp(theta[1]), dW = exp(theta[2]))
}

model <- form(dlmMLE(Nile, parm = c(1, 1), form)$par)
filtered <- dlmFilter(Nile, model)

autoplot(filtered)
```

```{r}
autoplot(filtered, ts.linetype = 'dashed', fitted.colour = 'blue')
```

```{r}
smoothed <- dlmSmooth(filtered)
class(smoothed)
autoplot(smoothed)
```

```{r}
p <- autoplot(filtered)
autoplot(smoothed, ts.colour = 'blue', p = p)
```

# KFAS包

```{r, message=FALSE, warning=FALSE}
library(KFAS)
model <- SSModel(
  Nile ~ SSMtrend(degree=1, Q=matrix(NA)), H=matrix(NA)
)
 
fit <- fitSSM(model=model, inits=c(log(var(Nile)),log(var(Nile))), method="BFGS")
smoothed <- KFS(fit$model)
autoplot(smoothed)
```

使用 `smoothing='none'` 可以画出过滤后的结果。

```{r}
filtered <- KFS(fit$model, filtering="mean", smoothing='none')
autoplot(filtered)
```

```{r}
trend <- signal(smoothed, states="trend")
class(trend)
```

```{r}
p <- autoplot(filtered)
autoplot(trend, ts.colour = 'blue', p = p)
```


# stats包

可支持的stats包里的对象有：

* `stl`, `decomposed.ts`
* `acf`, `pacf`, `ccf`
* `spec.ar`, `spec.pgram`
* `cpgram` 

```{r, message = FALSE}
autoplot(stl(AirPassengers, s.window = 'periodic'), ts.colour = 'blue')
```

```{r, fig.width = 4, fig.height = 2}
autoplot(acf(AirPassengers, plot = FALSE))
```

```{r, fig.width = 4, fig.height = 2}
autoplot(acf(AirPassengers, plot = FALSE), conf.int.fill = '#0000FF', conf.int.value = 0.8, conf.int.type = 'ma')
```

```{r, fig.width = 4, fig.height = 2}
autoplot(spec.ar(AirPassengers, plot = FALSE))
```

```{r, message = FALSE, fig.width=3, fig.height=3}
ggcpgram(arima.sim(list(ar = c(0.7, -0.5)), n = 50))
```

```{r, message = FALSE, fig.width = 5, fig.height = 5}
library(forecast)
ggtsdiag(auto.arima(AirPassengers))
```

```{r, message = FALSE, fig.width = 4, fig.height = 4}
gglagplot(AirPassengers, lags = 4)
```

更多关于时间序列的例子，请参考[Rpubs](http://rpubs.com/sinhrks/plot_ts)上的介绍。

最近又多里许多额外的非常好用的功能，比如说现在已经支持`multiplot`同时画多个不同对象，强烈推荐参考[Rpubs](http://rpubs.com/sinhrks/ggfortify_subplots) 以及关注我们[Github](https://github.com/sinhrks/ggfortify)上的更新。

祝大家使用愉快! 有问题请及时在Github上[submit an issue](https://github.com/sinhrks/ggfortify/issues)。(可以使用中文)






